function love.load()
	bind, control = love.filesystem.load("TLbind.lua")()
	require"TLfres"
	require"helper"
	require"content"
	
	math.randomseed(os.time())
	
	bind.useKeyboard, bind.useJoystick, bind.deadzone = true, true, .1
	bind.keys = {
			w="forward", a="left", s="back", d="right", lshift="run", escape="menu",		-- [" "]="jump"
			up="forward", left="left", down="back", right="right", rshift="run",
		}
	bind.joyAxes = { {"horiz", "vert"} }
	bind.joyBtns = { {"run", [7]="menu"} }
	bind.joyBalls = {}
	bind.joyHats = { {{"left","right","forward","back"}} }
	bind.maps = { horiz={"left","right"}, vert={"forward","back"} }
	bind.circleAnalogue = { {"horiz", "vert"} }
	
	TLfres.setScreen({}, 1, true)
	
	pc = {
		x=6,y=4, r=3.2, size=.3,
		vx=0,vy=0,
		i=graphics.terminator,
		toughness=.9999,	-- todo: toughness is subtracted from damage? It'd work like armor! Hm...
		health=1,
	}
	
	actors = { pc,
		{
			x=9.2,y=4.3, r=2, size=.3,
			vx=0,vy=0,
			i=graphics.punk,
			toughness=.3,
			health=1,
		}
	}
end



function love.keypressed(k)
	if k=="escape" then love.event.push("quit")
	elseif k=="r" then love.filesystem.load("main.lua")() love.load()
	elseif k=="f1" then debugMode = not debugMode
	end
end



function love.update(dt)
	dt = math.min(dt, .49)	-- ensure dt never gets too big.
	bind:update()
	
	do	-- pc stuff
		local s = control.run and 8 or 3
		pc.r = pc.r - control.horiz * dt*2
		pc.vx, pc.vy = pc.vx-(pc.vx-control.vert*math.sin(pc.r)*s) *dt*5, pc.vy-(pc.vy-control.vert*math.cos(pc.r)*s) *dt*5
		
		-- TODO: right-click to grab NPC, right-click again to throw
		
		-- npc test
		local punk = actors[2]
		if love.keyboard.isDown"z" then punk.vy = 15 end
		if love.keyboard.isDown"x" then punk.vx = 15 end
		punk.vx, punk.vy = punk.vx-(punk.vx-0)*dt*5, punk.vy-(punk.vy-0)*dt*5
	end
	
	do	-- physics
		dC = {}	-- debug: collision points
		
		local function charPhys( char )
			local g = map.ground	-- index of last ground tile type
			local s = char.speed or 2
			
			char.x, char.y = char.x+char.vx*dt, char.y+char.vy*dt
			
			
			-- pretty much point-circle collision
			local function collide( char, tx, ty )
				local cx,cy = math.clamp(-.5,char.x-tx,.5)+tx, math.clamp(-.5,char.y-ty,.5)+ty
				table.insert(dC, cx) table.insert(dC, cy)
				if (map[tx][ty] or 0) > g	-- there's a blocking tile
				and (map.damage[tx][ty] or 0) < 1	-- and it's not destroyed
				and math.dist(char.x, char.y, cx,cy ) < char.size then	-- and it's being collided with
					local t = map[tx][ty]
					
					-- damage calculations
					if not rawget(map.damage, tx) then map.damage[tx] = {} end
					local d = (( math.abs(char.vx)+math.abs(char.vy) )^2.5) * dt
					map.damage[tx][ty] = math.min((map.damage[tx][ty] or 0) + d*materials[t].resist, 1)
					char.health = char.health - d*(1-char.toughness)		-- subtract health
					char.vx, char.vy = char.vx*(1-d*.1), char.vy*(1-d*.1)	-- slow velocity by damage done
					
					-- debris
					-- TODO: take this quad method and apply it to tiles.
					local groundOffset = 1
					local quad = love.graphics.newQuad(0, 0, 8,8, 32,32)
					for i=1, d*materials[t].debrisRate do		-- debris rate!
						quad:setViewport(t*8-8, math.random(0,3)*8, 8,8)	-- choose random chunk for tile
						map.debris:addq(quad, tx-math.random()*2+1, ty-math.random()*2+1, math.random(7), 1/16, 1/16, 4,4)
					end
					
					-- push out of collision
					local nx, ny = math.normalize(char.x-cx, char.y-cy)
					char.x, char.y = cx+nx*char.size, cy+ny*char.size
				end
			end
		
			-- to collide with tiles, find nearest corner of the 4 nearest tiles
			local l,r,u,d = math.floor(char.x), math.ceil(char.x), math.floor(char.y),math.ceil(char.y)
			collide(char,l,u) collide(char,r,u) collide(char,l,d) collide(char,r,d)
			
		end
		for k,v in pairs(actors) do charPhys(v) end
		-- TODO: make actors collide
	end
end



function love.draw()
	local lg = love.graphics
	lg.push()
		TLfres.transform()
		
		lg.rotate(pc.r)
		lg.scale(.04, .04)
		lg.translate(-pc.x+math.sin(pc.r)*3, -pc.y+math.cos(pc.r)*3)
		
		local ts = 1/16
		local quad = love.graphics.newQuad(0, 0, 16,16, 64,64)
		for x=1,#map do for y=1,#map[x] do
			local df = 1	-- damage frame
			
			--lg.draw( graphics.tiles[map[x][y]][df], x,y, 0, ts,ts, 8,8)
			
			quad:setViewport( map[x][y]*16-16, math.floor((map.damage[x][y] or 0) * 3)*16, 16,16)	-- tile and damage
			lg.drawq( graphics.tiles, quad, x,y, 0, ts,ts, 8,8)
			
			if debugMode and rawget(map.damage[x],y) and map.damage[x][y]<1 then
				lg.print(math.round(map.damage[x][y],2), x,y,-pc.r,.015)
			end
		end end
		
		lg.draw( map.debris )
		
		-- lg.draw( graphics.terminator, pc.x,pc.y,-pc.r, ts,ts, 8,10 )
		for k,v in pairs(actors) do
			lg.draw( v.i, v.x,v.y,-v.r, ts,ts, 8,8 )
		end
		
		local l,r,u,d = math.floor(pc.x), math.ceil(pc.x), math.floor(pc.y),math.ceil(pc.y)
		
		-- debug: collision points
		if debugMode then lg.setColor(0,255,255) for i=1,#dC, 2 do lg.rectangle("fill", dC[i], dC[i+1], .06,.06) end end
		
	lg.pop()
	
	if debugMode then
		lg.setColor(255,255,255)
		D.PC.Coords = math.round(pc.x,2)..", "..math.round(pc.y,2)
		D.PC.Health = math.round(pc.health,2)
		drawdmessages()
	end
end